
/*

Copyright (c) 2018 Texas Instruments Incorporated

All rights reserved not granted herein.

Limited License.  

Texas Instruments Incorporated grants a world-wide, royalty-free, non-exclusive license under copyrights and patents it now or hereafter owns or controls to make, have made, use, import, offer to sell and sell ("Utilize") this software subject to the terms herein.  With respect to the foregoing patent license, such license is granted  solely to the extent that any such patent is necessary to Utilize the software alone.  The patent license shall not apply to any combinations which include this software, other than combinations with devices manufactured by or for TI (“TI Devices”).  No hardware patent is licensed hereunder.

Redistributions must preserve existing copyright notices and reproduce this license (including the above copyright notice and the disclaimer and (if applicable) source code license limitations below) in the documentation and/or other materials provided with the distribution

Redistribution and use in binary form, without modification, are permitted provided that the following conditions are met:

*	No reverse engineering, decompilation, or disassembly of this software is permitted with respect to any software provided in binary form.

*	any redistribution and use are licensed by TI for use only with TI Devices.

*	Nothing shall obligate TI to provide you with source code for the software licensed and provided to you in object code.

If software source code is provided to you, modification and redistribution of the source code are permitted provided that the following conditions are met:

*	any redistribution and use of the source code, including any resulting derivative works, are licensed by TI for use only with TI Devices.

*	any redistribution and use of any object code compiled from the source code and any resulting derivative works, are licensed by TI for use only with TI Devices.

Neither the name of Texas Instruments Incorporated nor the names of its suppliers may be used to endorse or promote products derived from this software without specific prior written permission.

DISCLAIMER.

THIS SOFTWARE IS PROVIDED BY TI AND TI’S LICENSORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL TI AND TI’S LICENSORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

*/

/*
 * gc global variable provides access to GUI Composer infrastructure components and project information.
 * For more information, please see the Working with Javascript guide in the online help.
 */
var gc = gc || {};
gc.services = gc.services || {};

var convertTemp = function(value, isToCelsius) {
    var result = value;
    if (isToCelsius) {
        result = (value - 32) / 1.8;
    } else {
        result = (value * 1.8) + 32;
    }
    return result;
};

var getF = function(value){
	return (value * 1.8) + 32;
};
var getC = function(value){
    return (value - 32) / 1.8;
};

var tempScale = "C";        // Holds current temperature Scale
var sampleNumber = 100;     // Holds current number of samples to plot
var sampleIndex = 0;        // Current Index of Sample to be placed in array
var sampleTimeArray = [];   // Array for time-samples

var saveText = function(fname, text) {
    var blob = new Blob([text], {
        type: 'text/plain;charset=utf-8'
    });
    saveAs(blob, fname);

    return false;
};

function convertTimestamp(timestamp) {
    var d = new Date(timestamp), // Convert the passed timestamp to milliseconds
        yyyy = d.getFullYear(),
        mm = ('0' + (d.getMonth() + 1)).slice(-2), // Months are zero based. Add leading 0.
        dd = ('0' + d.getDate()).slice(-2), // Add leading 0.
        hh = d.getHours(),
        h = hh,
        min = ('0' + d.getMinutes()).slice(-2), // Add leading 0.
        sec= ('0' + d.getSeconds()).slice(-2), // Add leading 0.
        msec= ('0' + d.getMilliseconds()).slice(-3), // Add leading 0.
        //ampm = 'AM',
        time;

    /*if (hh > 12) {
        h = hh - 12;
        ampm = 'PM';
    } else if (hh === 12) {
        h = 12;
        ampm = 'PM';
    } else if (hh === 0) {
        h = 12;
    }*/

    // ie: 2013-02-18, 8:35 AM	
    //time = +h + ':' + min + ' ' + ampm + ',' + yyyy + '-' + mm + '-' + dd;
    time = +h + ':' + min + ':' + sec + ':' + msec + ' ';// + ',' + yyyy + '-' + mm + '-' + dd;
    return time;
}

var getHex16 = function(value) {
    // convert the register value into a human readable format for the GUI and convert to F if needed
    var result = gc.utils.getHexString(value, 16, "0x");
    return result;
};

var extendAboutBox = function() {
    var aboutBox = document.querySelector('ti-widget-aboutbox');
    if (aboutBox){
        aboutBox.addEventListener('aboutbox_opening', function(event){
            aboutBox.appInfoTextHeading = 'Details';
            aboutBox.softwareManifestLink = "app/docs/TMP117EVM GUI_1.0.16_manifest.html";
            var str = '******************************\nTMP117EVM GUI\n******************************\n';
            str = str + 'Version                 : 1.2.1 ';
            str = str + 'Publish Date            : 04/18/2019\n';
            str = str + 'TMP117EVM_GUI v1.2.1\n';
            str = str + '\n';
            str = str + 'Change List\n';
            str = str + '  06/28/2018: Initial Released\n';
            str = str + '  10/01/2018: Additional configuration settings added to plot page. Reset bit added\n';
            str = str + '  10/08/2018: Register descriptions updated. Die Id made visible\n';
            str = str + '  10/12/2018: I2C General call reset & I2C frequency selection restored\n';
            str = str + '  11/05/2018: Save Chart Data reformatted, disabled register settings saving\n'
            str = str + '  11/20/2018: Temperature graph changed, sample number control added\n';
            str = str + '  04/18/2019: Added Code Generator option.\n';
            str = str + '\n';
            str = str + '******************************\nConnected device\n******************************\n';
            aboutBox.appInfoText = str + 'Retreiving version information ...\nPlease connect hardware';
            aboutBox.numRowsInAppInfoTextArea = 16;
        });
    }
};
/*
 *  Boilerplate code for creating computed data bindings
 */
document.addEventListener('gc-databind-ready', function() {
    /*
     *   Add custom computed value databindings here, using the following method:
     *
     *   function gc.databind.registry.bind(targetBinding, modelBinding, [getter], [setter]);
     *
     *   param targetBinding - single binding string or expression, or array of binding strings for multi-way binding.
     *   param modelBinding - single binding string or expression, or array of binding strings for multi-way binding.
     *   param getter - (optional) - custom getter function for computing the targetBinding value(s) based on modelBinding value(s).
     *   param setter - (optional) - custom setter function for computing the modelBinding value(s) based on targetBinding value(s).
     */

    // For example, a simple computed values based on simple expression
    // gc.databind.registry.bind('widget.id.propertyName', "targetVariable == 1 ? 'binding is one' : 'binding is not one'");

    // Or a custom two-way binding with custome getter and setter functions.  (setter is optional)  (getter only indicates one-way binding)
    // gc.databind.registry.bind('widget.id.propertyName', "targetVariable", function(value) { return value*5/9 + 32; }, function(value) { (return value-32)*9/5; });

    // Event 1 to n bindings
    /*
    gc.databind.registry.bind('widget.date.value',
        // dependant bindings needed in order to compute the date, in name/value pairs.
        {
            weekday: 'widget.dayOfWeek.selectedText',
            day: 'widget.dayOfMonth.value',
            month: 'widget.month.selectedText',
            year: 'widget.year.value'
        },
        // getter for date computation
        function(values)
        {
            // compute and return the string value to bind to the widget with id 'date'
            return values.weekday + ', ' + values.month + ' ' + values.day + ', ' + values.year;
        }
    );
	*/

	
	//Continous radio button on data page, 
	gc.databind.registry.bind('widget.ti_widget_radiobtn_continuous.checked','reg.CONFIG_REG.CONV_MODE',
        function(value) {
            // convert register value to state of a continous radio button  (i.e. checked property setting)
            var result = value;
            if ((value !== undefined) && (value !== null)) {
				if (value === 0 || value === 2) {
					result = true;
				}
				else if (value === 1 || value === 3) {
					result = false;
				}
            }
            return result;
        },
        function(value) {
            // If user enables continous mode, then write 0 to CONV_MODE bitfield
            var result = value;
            if ((value !== undefined) && (value !== null)) {
                if (value) {
                     result = 0;
                }
				else {
					result = 1;
				}
            }
            return result;
        }
    );
    
	
	//Shutdown radio button on data page, 
	gc.databind.registry.bind('widget.ti_widget_radiobtn_shutdown.checked','reg.CONFIG_REG.CONV_MODE',
        function(value) {
            // convert register value to state of a shutdown radio button  (i.e. checked property setting)
            var result = value;
            if ((value !== undefined) && (value !== null)) {
				if (value === 1 || value === 3) {
					result = true;
					// If shutdown mode enabled, then also enable triggering of Send One Shot button widget
					templateObj.$.ti_widget_button_sendoneshot.disabled = false;
				}
				else {
					result = false;
					templateObj.$.ti_widget_button_sendoneshot.disabled = true;
				}
            }
            return result;
        },
        function(value) {
            // If user enables shutdown mode, then write 1 to CONV_MODE bitfield
            var result = value;
            if ((value !== undefined) && (value !== null)) {
                if (value) {
                     result = 1;
                }
				else {
					result = 0;
				}
            }
            return result;
        }
    );
	
    //Alert High Limit spinner conversion
    gc.databind.registry.bind('widget.ti_widget_spinner_high_limit.value','reg.ALERT_HI_LIMIT.Alert_High_Limit',
        function(value) {
			if (tempScale === "C")
				return Number.parseFloat(value).toFixed(3);
			else 
				return Number.parseFloat(getF(value)).toFixed(3);
        },
        function(value) {
            if (tempScale === "C")
				return value;
			else 
				return getC(value);
        }
    );
  
  //Alert Low limit spinner conversion
   gc.databind.registry.bind('widget.ti_widget_spinner_low_limit.value','reg.ALERT_LO_LIMIT.Alert_Low_Limit',
        function(value) {
			if (tempScale === "C")
				return Number.parseFloat(value).toFixed(3);
			else 
				return Number.parseFloat(getF(value)).toFixed(3);
        },
        function(value) {
            if (tempScale === "C")
				return value;
			else 
				return getC(value);
        }
    );
    
    //Offset temperature Conversion
    gc.databind.registry.bind('widget.ti_widget_spinner_offset_temp.value','reg.OFFSET_TEMP.OFFSETTEMP',
        function(value) {
			if (tempScale === "C")
				return Number.parseFloat(value).toFixed(3);
			else 
				return Number.parseFloat(getF(value)).toFixed(3);
        },
        function(value) {
            if (tempScale === "C")
				return value;
			else 
				return getC(value);
        }
    );

	//Temperature Result conversion
    gc.databind.registry.bind('widget.ti_widget_label_temp.label','reg.TEMP_RESULT.TEMP',
        function(value) {
            if (tempScale === "C")
				return Number.parseFloat(value).toFixed(3);
			else 
				return Number.parseFloat(getF(value)).toFixed(3);
        }
    );
    
    //Databinding for Series 0 on Scalar Graph, including conversion handling and timestamp recording
    gc.databind.registry.bind('ti_widget_scalargraph_temp.value_0', 'my_model.TEMP_RESULT.TEMP',
        function(value){
            var timestamp = new Date();
            if(sampleIndex < sampleNumber )
            {
                sampleTimeArray.push(timestamp);
                sampleIndex += 1;
            }
            else
            {
                sampleTimeArray.shift();
                sampleTimeArray.push(timestamp);
            }

            
            if (tempScale === "C")
                return value; // Number.parseFloat(value).toFixed(3);
            else
                return getF(value); //Number.parseFloat(getF(value).toFixed(3));
        });
    
    /* This binds the auto-read selector from plot page to auto-read selector on register page. Can't be done simply
       because the necessary properties of the autoReadDroplist on the register page are abstacted from interaction.
    */
    gc.databind.registry.bind('ti_widget_droplist_auto_read.selectedIndex', "ti_widget_register_page.autoReadValue", 
        function(value){                                // Getter function, sets drop list based on autoReadValue changes
            switch(Number(value)){
                case -1: //Auto read is off
                    return 0;
                case 0: // As Fast as possible
                    return 1;
                case 15: // 15ms delay
                    return 2;
                case 50: // 50mS delay
                    return 3;
                case 100: // 100ms delay
                    return 4;
                case 500: // 500mS delay
                    return 5;
                case 1000: // 1000mS delay
                    return 6;
                default:
                    return 4;
            }
        },
        function(value){                                // Setter function, sets autoReadValue based on droplist chnages
            switch(Number(value)){
                case 0: //Auto read is off
                    return -1;
                case 1: // As Fast as possible
                    return 0;
                case 2: // 15ms delay
                    return 15;
                case 3: // 50mS delay
                    return 50;
                case 4: // 100ms delay
                    return 100;
                case 5: // 500mS delay
                    return 500;
                case 6: // 1000mS delay
                    return 1000;
                default:
                    return 100;
            }
        });
    
    //Set tick period to match conversion cycle, use widget to widget binding.
    //NOT WORKING AS EXPECTED. Update time of graph is unpredictable. Best to leave tickOn as false for now
    /*gc.databind.registry.bind('ti_widget_scalargraph_temp.tickPeriod', "ti_widget_droplist_convcycle_plot.selectedValue", 
        function(value){
            switch(Number(value)){
                case 0: //15.5mS conversion Cycle
                    return 0;
                case 1: // 125mS Conversion Cycle
                    return 125;
                case 2: //250mS Conversion cycle
                    return 250;
                case 3: //500mS Conversion Cycle
                    return 500;
                case 4: //1S Conversion Cycle
                    return 1000;
                case 5: //4S Conversion Cycle
                    return 4000;
                case 6: //8S Conversion Cycle
                    return 8000;
                case 7: //16S Conversion Cycle
                    return 16000;
                default:
                    return 0;
            }
        });*/
	
	//function gc.databind.registry.bind(targetBinding, modelBinding, [getter], [setter]);

	gc.databind.registry.bind('widget.ti_widget_label_did.label', 'reg.ID_REG.DID',getHex16);
	extendAboutBox();
});

/*
 *  Boilerplate code for creating custom actions
 */

document.addEventListener('gc-nav-ready', function() {
    /*
     *   Add custom actions for menu items using the following api:
     *
     *   function gc.nav.registryAction(id, runable, [isAvailable], [isVisible]);
     *
     *   param id - uniquely identifies the action, and should correspond to the action property of the menuaction widget.
     *   param runable - function that performs the custom action.
     *   param isAvailable - (optional) - function called when the menu action is about to appear.  Return false to disable the action, or true to enable it.
     *   param isVisible - (optional) - function called when the menu action is about to appear.  Return false to hide the action, or true to make it visible.
     */

    // For example,
    // gc.nav.registerAction('myCustomCloseAction', function() { window.close(); }, function() { return true; }, function() { return true; });

    // Alternatively, to programmatically disable a menu action at any time use:
    // gc.nav.disableAction('myCustomCloseAction);    then enable it again using:  gc.nav.enableAction('myCustomCloseAction');

    gc.nav.registerAction('show_introduction', function() {
        if ((templateObj) && (templateObj.$)) {
            templateObj.$.ti_widget_vtabcontainer.selectedIndex = 0;
            templateObj.$.ti_widget_menuitem.hideFullMenu();
        }
    }, function() {
        return true;
    }, function() {
        return true;
    });
    gc.nav.registerAction('show_registers', function() {
        if ((templateObj) && (templateObj.$)) {
            templateObj.$.ti_widget_vtabcontainer.selectedIndex = 1;
            templateObj.$.ti_widget_menuitem.hideFullMenu();
        }
    }, function() {
        return true;
    }, function() {
        return true;
    });
    gc.nav.registerAction('show_motor_control', function() {
        if ((templateObj) && (templateObj.$)) {
            templateObj.$.ti_widget_vtabcontainer.selectedIndex = 2;
            templateObj.$.ti_widget_menuitem.hideFullMenu();
        }
    }, function() {
        return true;
    }, function() {
        return true;
    });

    gc.nav.registerAction('load_registers', function() {
        if ((templateObj) && (templateObj.$)) {
            gc.File.browseAndLoad(null, {
                bin: false
            }, function(contents, properties) {
                var regData = null;
                try {
                    regData = JSON.parse(contents);
                    if (regData) {
                        templateObj.setRegisterData(regData);
                        templateObj.$.ti_widget_eventlog_view.log("info", "Load Registers: loaded from :" + properties.localPath);
                    }
                } catch (ex) {
                    templateObj.logMsgAndNotify("error", "Load Registers: Could not parse file:" + ex, false, true);
                }

            });
        }
    }, function() {
        return true;
    }, function() {
        return true;
    });

    var saveText = function(fname, text, mime) {
        var blob = new Blob([text], {
            type: mime
        });
        saveAs(blob, fname);
        return false;
    };
    gc.nav.registerAction('save_registers', function() {
        if ((templateObj) && (templateObj.$)) {
            var regData = templateObj.getRegisterData();
            var regDataJson = JSON.stringify(regData);
            if (gc.desktop.isDesktop()) {
                gc.File.browseAndSave(regDataJson, 'regs.json', '*.json', function(result, errorInfo) {
                    if (errorInfo) {
                        templateObj.logMsgAndNotify("error", "Save Registers: " + errorInfo, false, true);
                    } else if ((result) && (result.localPath)) {
                        templateObj.$.ti_widget_eventlog_view.log("info", "Save Registers: saved to " + result.localPath);
                    }

                });
            } else {
                saveText("regs.json", regDataJson, 'text/plain;charset=utf-8');
                templateObj.$.ti_widget_eventlog_view.log("info", "Save Registers: regs.json file downloaded");
            }
        }
    }, function() {
        return true;
    }, function() {
        return true;
    });

    gc.nav.registerAction('save_registers_as', function() {
        if ((templateObj) && (templateObj.$)) {
            var regData = templateObj.getRegisterData();
            var regDataJson = JSON.stringify(regData);
            if (gc.desktop.isDesktop()) {
                gc.File.browseAndSave(regDataJson, 'regs.json', '*.json', function(result, errorInfo) {
                    if (errorInfo) {
                        templateObj.logMsgAndNotify("error", "Save Registers As: " + errorInfo, false, true);
                    } else if ((result) && (result.localPath)) {
                        templateObj.$.ti_widget_eventlog_view.log("info", "Save Registers As: saved to " + result.localPath);
                    }
                });
            } else {
                saveText("regs.json", regDataJson, 'text/plain;charset=utf-8');
                templateObj.$.ti_widget_eventlog_view.log("info", "Save Registers: regs.json file downloaded");
            }
        }
    }, function() {
        return true;
    }, function() {
        return true;
    });

    gc.nav.registerAction('open_log_pane', function() {
        if ((templateObj) && (templateObj.$)) {
            templateObj.$.ti_widget_eventlog_view.openView();
        }
    }, function() {
        return true;
    }, function() {
        return true;
    });

});

/*
 *  Boilerplate code for working with components in the application gist
 */


var initComplete = false;
var templateObj;
var registerModel;
var graphWidgetTempScale;


// Wait for DOMContentLoaded event before trying to access the application template
var init = function() {
    templateObj = document.querySelector('#template_obj');
    // Wait for the template to fire a dom-change event to indicate that it has been 'stamped'
    // before trying to access components in the application.
    templateObj.addEventListener('dom-change', function() {
        if (initComplete) return;
        this.async(function() {
            initComplete = true;
            console.log("Application template has been stamped.");
            templateObj.$.ti_widget_toast.hideToast();
            templateObj.$.ti_widget_eventlog_view.log("info", "Application started.");
            registerModel = Polymer.dom(document).querySelector('ti-model-register');
			
			var graphWidgetTempScale = gc.databind.registry.getBinding('widget.ti_widget_temperaturegraph.temperatureScale'); 
			
            registerModel.addEventListener("usbHidModelReceivingData", function(event) {
                //templateObj.logMsgAndNotify("debug", "Receiving data from target.", false, false);
            });
            templateObj.$.ti_widget_temperaturegraph.addEventListener("graphReady", function(event) {
                console.log("graph ready event received");
            });

            // Now that the template has been stamped, you can use 'automatic node finding' $ syntax to access widgets.
            // e.g. to access a widget with an id of 'widget_id' you can use templateObj.$.widgetId

            //templateObj.$.ti_widget_menubar.productName = templateObj.$.ti_widget_register_page.deviceName;

            templateObj.updateTempScale = function() {
                var origScale = tempScale;
                if (ti_widget_radio_group_scale.selectedText.toLowerCase() === "farenheit") {
                    tempScale = "F";
                } else {
                    tempScale = "C";
                }
                if (origScale !== tempScale) {
                    templateObj.$.ti_widget_temperaturegraph.reset();
                    templateObj.$.ti_widget_scalargraph_temp.reset(); //reset scalar graph
                    sampleTimeArray = [];
                    sampleIndex = 0;
                    //templateObj.$.ti_widget_temperaturegraph.temperatureScale = tempScale;
					graphWidgetTempScale.setValue(tempScale);
                    var isCelsius = (tempScale === "C");
                    templateObj.$.ti_widget_spinner_high_limit.value = convertTemp(templateObj.$.ti_widget_spinner_high_limit.value, isCelsius);
                    templateObj.$.ti_widget_spinner_low_limit.value = convertTemp(templateObj.$.ti_widget_spinner_low_limit.value, isCelsius);
                    templateObj.$.ti_widget_spinner_offset_temp.value = convertTemp(templateObj.$.ti_widget_spinner_offset_temp.value, isCelsius);
                }

            };

            templateObj.updateTempScale();

            templateObj.$.ti_widget_radio_group_scale.addEventListener("changed", function(event) {
                templateObj.updateTempScale();
            });
            
            templateObj.$.ti_widget_spinner_high_limit_cfg.addEventListener("changed", function(event) {
                templateObj.$.ti_widget_code_high_limit.checked = "true";
            });

            templateObj.$.ti_widget_spinner_high_limit.addEventListener("changed", function(event) {
                templateObj.$.ti_widget_code_high_limit.checked = "true";
            });

            templateObj.$.ti_widget_spinner_low_limit_cfg.addEventListener("changed", function(event) {
                templateObj.$.ti_widget_code_low_limit.checked = "true";
            });
            
            templateObj.$.ti_widget_spinner_low_limit.addEventListener("changed", function(event) {
                templateObj.$.ti_widget_code_low_limit.checked = "true";
            });
            
            templateObj.$.ti_widget_spinner_eeprom1.addEventListener("changed", function(event) {
                templateObj.$.ti_widget_code_eeprom1.checked = "true";
            });

            templateObj.$.ti_widget_spinner_eeprom2.addEventListener("changed", function(event) {
                templateObj.$.ti_widget_code_eeprom2.checked = "true";
            });

            templateObj.$.ti_widget_spinner_eeprom3.addEventListener("changed", function(event) {
                templateObj.$.ti_widget_code_eeprom3.checked = "true";
            });

            templateObj.$.ti_widget_spinner_offset.addEventListener("changed", function(event) {
                templateObj.$.ti_widget_code_temp_offset.checked = "true";
            });

            templateObj.$.ti_widget_spinner_offset_temp.addEventListener("changed", function(event) {
                templateObj.$.ti_widget_code_temp_offset.checked = "true";
            });

            templateObj.$.ti_widget_label_learnmore.addEventListener("click",function(event){
                 templateObj.$.ti_widget_label_productname.fontColor="#444";
                 templateObj.$.ti_widget_label_productdesc.fontColor="#444";
                 templateObj.$.ti_widget_label_learnmore.setAttribute('hidden',true);
                 templateObj.$.ti_widget_label_showless.removeAttribute('hidden');
                 templateObj.$.ti_widget_vtab_home.style.backgroundColor = "white";
                 templateObj.$.ti_widget_tabcontainer.selectedIndex = 1;
            });
            templateObj.$.ti_widget_label_showless.addEventListener("click",function(event){
                 templateObj.$.ti_widget_label_productname.fontColor="white";
                 templateObj.$.ti_widget_label_productdesc.fontColor="white";
                 templateObj.$.ti_widget_label_learnmore.removeAttribute('hidden');
                 templateObj.$.ti_widget_label_showless.setAttribute('hidden',true);
                 templateObj.$.ti_widget_vtab_home.style.backgroundColor = "rgb(20, 140, 156)";
                 templateObj.$.ti_widget_tabcontainer.selectedIndex = 0;
            });
            if (templateObj.$.ti_widget_button_connect) {
                templateObj.$.ti_widget_button_connect.addEventListener("click",function(event){
                    if (gc.connectionManager){
                        gc.connectionManager.connect();
                    }
                });
	        }
			
			templateObj.updateMode = function() {
                    var bind = gc.databind.registry.getBinding('reg.CONFIG_REG.CONV_MODE')
					if (templateObj.$.ti_widget_radio_group_mode.selectedText.toLowerCase() === "continuous") {
							bind.setValue(0); 
							templateObj.$.ti_widget_button_sendoneshot.disabled = true;
					} else {
                            bind.setValue(1);
							templateObj.$.ti_widget_button_sendoneshot.disabled = false;
					}
				};
 //           templateObj.updateMode();
                                                
			templateObj.$.ti_widget_button_sendoneshot.addEventListener("click", function(event) {
					var bind = gc.databind.registry.getBinding('reg.CONFIG_REG.CONV_MODE');
					         bind.setValue(1);
					         bind.setValue(3);
            });
            templateObj.$.ti_widget_droplist_sample_number.addEventListener("selected", function(event) {
                sampleNumber = Number(templateObj.$.ti_widget_droplist_sample_number.selectedText)
                sampleTimeArray = [];
                sampleIndex = 0;
            });
                                                
            
        //            In the attached function for the graph, add:
        templateObj.$.ti_widget_vtabcontainer.addEventListener("selected_tab_changed",function(event){
             templateObj.$.ti_widget_temperaturegraph.asyncInitOptions(true);
             templateObj.async(templateObj.$.ti_widget_temperaturegraph.onResize,1);
        });

        //Clear chart and reset sampleTimeArray
            templateObj.$.ti_widget_button_clearchart.addEventListener("click", function(event) {
                templateObj.$.ti_widget_temperaturegraph.reset();
                templateObj.$.ti_widget_scalargraph_temp.reset();
                sampleTimeArray = [];
                sampleIndex = 0;
            });
            
            //Save Chart for Scalar Graph
            templateObj.$.ti_widget_button_savechart.addEventListener("click", function(event) {
                
                //Want to print date of sample data next to heading row, need to define new date to do so.
                var today = new Date();
                var dd = today.getDate();
                var mm = today.getMonth()+1; //January is 0!
                var yyyy = today.getFullYear();
                
                if(dd<10) {
                    dd = '0'+dd
                } 
                
                if(mm<10) {
                    mm = '0'+mm
                } 
                
                today = mm + '/' + dd + '/' + yyyy;
                //End date creation
                //Offset Register Value in C
                var offsetRegValue = gc.databind.registry.getBinding('reg.OFFSET_TEMP').getValue()/128;
                //Check if we are in F, convert Offset reg value if so.
                offsetRegValue = (tempScale == 'C') ? offsetRegValue : getF(offsetRegValue);
                
                //Print Header information for .csv file to result string
                var result = 'Config Reg: 0x' + gc.databind.registry.getBinding('reg.CONFIG_REG').getValue().toString(16);                      // Config Register Value
                result += ",Read Delay: " + templateObj.$.ti_widget_register_page.autoReadValue;                                                // Read Delay Value
                result += "ms, Offset Register value:["+ offsetRegValue + tempScale;         // Offset Register value
                result += "],I2C Address: " + templateObj.$.ti_widget_droplist_i2c_addr.selectedText;                                           // I2C Address
                result +=  ",Device ID: "+ templateObj.$.ti_widget_label_did.label + "\n";                                                      // Device ID
                result += "Sample #,Temperature [" + tempScale + "],Time, Date: " + today + "\n";                                               // Label Row for data + Date
                
                //strData0 holds values from scalar graph for series 0
                var strData0 = templateObj.$.ti_widget_scalargraph_temp.series_0_values;
                for (var i = 0; i < strData0.length; i++) {
                    var timestamp = convertTimestamp(sampleTimeArray[i]);
                    //result = result + i + "," + strData0[i] + "\n"
                    result = result + i + "," + strData0[i] + ',' + timestamp + "\n";
                }
                
                //Save Excel File
                saveText("tempChart.csv", result);
            });

            // open the eventlog viewer when the user clicks on the book icon in the status bar:
            templateObj.$.ti_widget_eventlog_view.closeView(); // ensure that view states are in sync with application
            templateObj.$.ti_widget_statusbar.addEventListener("appiconclicked", function(event) {
                templateObj.$.ti_widget_eventlog_view.toggleView();

            });

            // demo of handling when the user clicks the Write Register button
            templateObj.$.ti_widget_register_page.addEventListener("write_reg", function(event) {
                var detailsObj = event.detail;
                templateObj.logMsgAndNotify(detailsObj.statusType, detailsObj.statusMsg, false, true);
            });
           
            templateObj.$.ti_widget_button_fw.addEventListener("click", function(event) {
                templateObj.$.ti_widget_label13_fw.label = templateObj.$.my_model.getCodec().version;
            });
            
            //Implement I2C Reset using u2a.sendCommandPacket
            templateObj.$.ti_widget_button_I2C_reset.addEventListener("click", function(event) {
               var u2a= document.querySelector('ti-model-register').getCodec();
               var Cmd_I2C_Write = 2;
               var commandSent = u2a.sendCommandPacket(Cmd_I2C_Write, [0, 0, 1, 6])
            }); 
            
            //Set I2C Frequency based on droplist
            templateObj.$.ti_widget_droplist2_i2c_freq.addEventListener("selected", function(event) {
               var u2a= document.querySelector('ti-model-register').getCodec();         // get codec
               var Cmd_I2C_Set = 1;                                                     // Command to set I2C port
               var speed = templateObj.$.ti_widget_droplist2_i2c_freq.selectedIndex;    // Speed based on droplist index: 0:100kHz, 1:400kHz, 2:10kHz
               var commandSent = u2a.sendCommandPacket(Cmd_I2C_Set, [speed, 0, 1])      // send command to set I2C, see packet definition in USB2ANY.h
            });
            
            
            //Implement software reset using binding to soft_reset
            templateObj.$.ti_widget_button_soft_reset.addEventListener("click", function(event) {
               var bind = gc.databind.registry.getBinding('reg.CONFIG_REG.SOFT_RESET');
               bind.setValue(1);
            }); 
            
            //software reset button on plot tab
            templateObj.$.ti_widget_button_soft_reset_plot.addEventListener("click", function(event) {
               var bind = gc.databind.registry.getBinding('reg.CONFIG_REG.SOFT_RESET');
               bind.setValue(1);
            });
            
            //Software reset button on configuration tab
            templateObj.$.ti_widget_button_soft_reset_config.addEventListener("click", function(event) {
               var bind = gc.databind.registry.getBinding('reg.CONFIG_REG.SOFT_RESET');
               bind.setValue(1);
            }); 
            
            //Software reset button on configuration tab
            templateObj.$.ti_widget_button_gen_code.addEventListener("click", function(event) {
                
                var result_c = '//\n// Include Generic Header Files Here\n//\n';
                
                result_c += '#include <stdint.h>\n';
                result_c += '#include <stdbool.h>\n';
                result_c += '#include "tmp117.h"\n\n';

                result_c += '//\n// Include User Header Files Here\n//\n\n';

                result_c += '\n/**************** MCU SPECIFIC HAL CODE STARTS HERE ****************/';
                result_c += '\nvoid sensor_write(unsigned long slaveaddress,';
                result_c += '\n                  unsigned long registerpointer,';
                result_c += '\n                  unsigned long registerdata)';
                result_c += '\n{';
                result_c += '\n //\n // Add the I2C Transmit API to write registerdata to slaveaddress pointed by registerpointer\n //\n';
                result_c += '\n}\n';

                result_c += '\nunsigned long sensor_read(unsigned long slaveaddress,';
                result_c += '\n                          unsigned long registerpointer)';
                result_c += '\n{';
                result_c += '\n unsigned long ulTemp = 0x0;';
                result_c += '\n //\n // Add the I2C Receive API to read registerdata from slaveaddress pointed by registerpointer\n //\n';
                result_c += '\n return(ulTemp);';
                result_c += '\n}';
                result_c += '\n/**************** MCU SPECIFIC HAL CODE ENDS HERE ****************/\n';

                result_c += '\nvoid tmp117_config(void)\n{\n';

                if(templateObj.$.ti_widget_commit_eeprom.checked) {
                    result_c += ' unsigned long ulCfg = 0x0;\n';
                }

                if(templateObj.$.ti_widget_commit_eeprom.checked) {
                    result_c += '\n //\n // EEPROM Register Write to Unlock\n //';
                    result_c += '\n sensor_write(TMP117_0_SLAVE_ADDRESS, TMP117_EEPROMUL_REG, EEPROMUL_UNLOCK);\n';
                }

                if(templateObj.$.ti_widget_code_config.checked) {
                    result_c += '\n //\n // Configuration Register Write\n //\n';
                    result_c += ' sensor_write(TMP117_0_SLAVE_ADDRESS, TMP117_CONFIG_REG, TMP117_0_CONFIG_REG);\n';
                    if(templateObj.$.ti_widget_commit_eeprom.checked) {
                        result_c += ' do {\n';
                        result_c += '     //\n     // EEPROM Register Read to check for Busy flag\n     //\n';
                        result_c += '     ulCfg = sensor_read(TMP117_0_SLAVE_ADDRESS, TMP117_EEPROMUL_REG);\n';
                        result_c += ' } while((ulCfg && EEPROMUL_BUSY) != EEPROMUL_BUSY);\n';
                    }
                }                

                if(templateObj.$.ti_widget_code_high_limit.checked) {
                    result_c += '\n //\n // High Limit Register Write\n //\n';
                    result_c += ' sensor_write(TMP117_0_SLAVE_ADDRESS, TMP117_HIGHLIMIT_REG, TMP117_0_HI_LIMIT_OFFSET);\n';
                    if(templateObj.$.ti_widget_commit_eeprom.checked) {
                        result_c += ' do {\n';
                        result_c += '     //\n     // EEPROM Register Read to check for Busy flag\n     //\n';
                        result_c += '     ulCfg = sensor_read(TMP117_0_SLAVE_ADDRESS, TMP117_EEPROMUL_REG);\n';
                        result_c += ' } while((ulCfg && EEPROMUL_BUSY) != EEPROMUL_BUSY);\n';
                    }
                }                

                if(templateObj.$.ti_widget_code_low_limit.checked) {
                    result_c += '\n //\n // Low Limit Register Write\n //\n';
                    result_c += ' sensor_write(TMP117_0_SLAVE_ADDRESS, TMP117_LOWLIMIT_REG, TMP117_0_LO_LIMIT_OFFSET);\n';
                    if(templateObj.$.ti_widget_commit_eeprom.checked) {
                        result_c += ' do {\n';
                        result_c += '     //\n     // EEPROM Register Read to check for Busy flag\n     //\n';
                        result_c += '     ulCfg = sensor_read(TMP117_0_SLAVE_ADDRESS, TMP117_EEPROMUL_REG);\n';
                        result_c += ' } while((ulCfg && EEPROMUL_BUSY) != EEPROMUL_BUSY);\n';
                    }
                }                

                if(templateObj.$.ti_widget_code_eeprom1.checked) {
                    result_c += '\n //\n // EEPROM1 Register Write\n //\n';
                    result_c += ' sensor_write(TMP117_0_SLAVE_ADDRESS, TMP117_EEPROM1_REG, TMP117_0_EEPROM1);\n';
                    if(templateObj.$.ti_widget_commit_eeprom.checked) {
                        result_c += ' do {\n';
                        result_c += '     //\n     // EEPROM Register Read to check for Busy flag\n     //\n';
                        result_c += '     ulCfg = sensor_read(TMP117_0_SLAVE_ADDRESS, TMP117_EEPROMUL_REG);\n';
                        result_c += ' } while((ulCfg && EEPROMUL_BUSY) != EEPROMUL_BUSY);\n';
                    }
                }                

                if(templateObj.$.ti_widget_code_eeprom2.checked) {
                    result_c += '\n //\n // EEPROM2 Register Write\n //\n';
                    result_c += ' sensor_write(TMP117_0_SLAVE_ADDRESS, TMP117_EEPROM2_REG, TMP117_0_EEPROM2);\n';
                    if(templateObj.$.ti_widget_commit_eeprom.checked) {
                        result_c += ' do {\n';
                        result_c += '     //\n     // EEPROM Register Read to check for Busy flag\n     //\n';
                        result_c += '     ulCfg = sensor_read(TMP117_0_SLAVE_ADDRESS, TMP117_EEPROMUL_REG);\n';
                        result_c += ' } while((ulCfg && EEPROMUL_BUSY) != EEPROMUL_BUSY);\n';
                    }
                }                

                if(templateObj.$.ti_widget_code_temp_offset.checked) {
                    result_c += '\n //\n // Temperature Offset Register Write\n //\n';
                    result_c += ' sensor_write(TMP117_0_SLAVE_ADDRESS, TMP117_TEMPOFFSET_REG, TMP117_0_TEMP_OFFSET);\n';
                    if(templateObj.$.ti_widget_commit_eeprom.checked) {
                        result_c += ' do {\n';
                        result_c += '     //\n     // EEPROM Register Read to check for Busy flag\n     //\n';
                        result_c += '     ulCfg = sensor_read(TMP117_0_SLAVE_ADDRESS, TMP117_EEPROMUL_REG);\n';
                        result_c += ' } while((ulCfg && EEPROMUL_BUSY) != EEPROMUL_BUSY);\n';
                    }
                }                

                if(templateObj.$.ti_widget_code_eeprom3.checked) {
                    result_c += '\n //\n // EEPROM3 Register Write\n //\n';
                    result_c += ' sensor_write(TMP117_0_SLAVE_ADDRESS, TMP117_EEPROM3_REG, TMP117_0_EEPROM3);\n';
                    if(templateObj.$.ti_widget_commit_eeprom.checked) {
                        result_c += ' do {\n';
                        result_c += '     //\n     // EEPROM Register Read to check for Busy flag\n     //\n';
                        result_c += '     ulCfg = sensor_read(TMP117_0_SLAVE_ADDRESS, TMP117_EEPROMUL_REG);\n';
                        result_c += ' } while((ulCfg && EEPROMUL_BUSY) != EEPROMUL_BUSY);\n';
                    }
                }                

                if(templateObj.$.ti_widget_commit_eeprom.checked) {
                    result_c += '\n //\n // EEPROM Register Write to Lock the NVM\n //\n';
                    result_c += ' sensor_write(TMP117_0_SLAVE_ADDRESS, TMP117_EEPROMUL_REG, EEPROMUL_LOCK);\n';
                }
                
                result_c += '\n return;\n}\n';
                
                result_c += '\nunsigned long tmp117_read(unsigned long sensorid)\n{\n unsigned long ulTemp = 0x0;\n';
                result_c += ' unsigned long ulCfg = 0x0;\n';
                if (ti_widget_droplist2.selectedText.toLowerCase() === "shutdown") {
                    result_c += '\n //\n // Configuration Register Read-modify-write\n';
                    result_c += ' // Read the value to ulCfg variable\n //\n';
                    result_c += ' ulCfg = sensor_read(TMP117_0_SLAVE_ADDRESS, TMP117_CONFIG_REG);\n';
                    result_c += ' ulCfg |= CONFIG_MOD_ONESHOT;\n';
                    result_c += ' sensor_write(TMP117_0_SLAVE_ADDRESS, TMP117_CONFIG_REG, ulCfg);\n';
                    result_c += '\n ulCfg = 0x0;\n';
                }
                result_c += ' do {\n';
                result_c += '     //\n     // A Configuration Register Read to check for Data ready flag\n     //\n';
                result_c += '     ulCfg = sensor_read(TMP117_0_SLAVE_ADDRESS, TMP117_CONFIG_REG);\n';
                result_c += '\n } while((ulCfg && CONFIG_DATAREADY) != CONFIG_DATAREADY);\n';
                result_c += '\n //\n // Temperature Register Read\n //\n';
                result_c += ' ulTemp = sensor_read(TMP117_0_SLAVE_ADDRESS, TMP117_TEMP_REG);\n';
                result_c += ' return(ulTemp);\n';
                result_c += '}\n';
                
                saveText("tmp117.c", result_c);
                
                result_c = '/* Sensor register specific defines */\n';
                result_c += '#define TMP117_TEMP_REG       0x00\n';
                result_c += '#define TMP117_CONFIG_REG     0x01\n';
                result_c += '#define TMP117_HIGHLIMIT_REG  0x02\n';
                result_c += '#define TMP117_LOWLIMIT_REG   0x03\n';
                result_c += '#define TMP117_EEPROMUL_REG   0x04\n';
                result_c += '#define TMP117_EEPROM1_REG    0x05\n';
                result_c += '#define TMP117_EEPROM2_REG    0x06\n';
                result_c += '#define TMP117_TEMPOFFSET_REG 0x07\n';
                result_c += '#define TMP117_EEPROM3_REG    0x08\n';
                result_c += '#define TMP117_DEVICEID_REG   0x0F\n';

                result_c += '#define CONFIG_DATAREADY   0x2000\n';
                result_c += '#define CONFIG_MOD_ONESHOT 0x0C00\n';
                result_c += '#define EEPROMUL_BUSY      0x4000\n';
                result_c += '#define EEPROMUL_UNLOCK    0x8000\n';
                result_c += '#define EEPROMUL_LOCK      0x0000\n';
                
                result_c += '\n/* Sensor instance slave address define */\n';
                result_c += '#define TMP117_0_SLAVE_ADDRESS ' + templateObj.$.ti_widget_droplist_i2c_addr.selectedText + '\n\n';

                result_c += '/* Sensor instance specific define */\n';
                if(templateObj.$.ti_widget_code_config.checked) {
                    var config_c = '// Configuration Setting\n';
                    var config_setting_c = '#define TMP117_0_CONFIG_REG 0x0';
                    if (ti_widget_droplist2.selectedText.toLowerCase() === "continuous conversion") {
                        config_c += '// Continuous Conversion\n';
                    } else if (ti_widget_droplist2.selectedText.toLowerCase() === "shutdown") {
                        config_c += '// One-Shot Conversion\n';
                        config_setting_c += '| 0x0400';
                    }
                
                    if (ti_widget_droplist_convcycle.selectedText.toLowerCase() === "15.5ms") {
                        config_c += '// Conversion cycle of 15.5 ms\n';
                    } else if (ti_widget_droplist_convcycle.selectedText.toLowerCase() === "125ms") {
                        config_c += '// Conversion cycle of 125 ms\n';
                        config_setting_c += '| 0x0080';
                    } else if (ti_widget_droplist_convcycle.selectedText.toLowerCase() === "250ms") {
                        config_c += '// Conversion cycle of 250 ms\n';
                        config_setting_c += '| 0x0100';
                    } else if (ti_widget_droplist_convcycle.selectedText.toLowerCase() === "500ms") {
                        config_c += '// Conversion cycle of 500 ms\n';
                        config_setting_c += '| 0x0180';
                    } else if (ti_widget_droplist_convcycle.selectedText.toLowerCase() === "1s") {
                        config_c += '// Conversion cycle of 1 sec\n';
                        config_setting_c += '| 0x0200';
                    } else if (ti_widget_droplist_convcycle.selectedText.toLowerCase() === "4s") {
                        config_c += '// Conversion cycle of 4 sec\n';
                        config_setting_c += '| 0x0280';
                    } else if (ti_widget_droplist_convcycle.selectedText.toLowerCase() === "8s") {
                        config_c += '// Conversion cycle of 8 sec\n';
                        config_setting_c += '| 0x0300';
                    } else if (ti_widget_droplist_convcycle.selectedText.toLowerCase() === "16s") {
                        config_c += '// Conversion cycle of 16 sec\n';
                        config_setting_c += '| 0x0380';
                    }
                
                    if (ti_widget_droplist_conv_avg.selectedText.toLowerCase() === "no Averaging (15.5ms in cc)") {
                        config_c += '// Averaging : None\n';
                    } else if (ti_widget_droplist_conv_avg.selectedText.toLowerCase() === "8 avg / 125ms in cc") {
                        config_c += '// Averaging : 8 Averages\n';
                        config_setting_c += '| 0x0020';
                    } else if (ti_widget_droplist_conv_avg.selectedText.toLowerCase() === "32 avg / 500ms in cc") {
                        config_c += '// Averaging : 32 Averages\n';
                        config_setting_c += '| 0x0040';
                    } else if (ti_widget_droplist_conv_avg.selectedText.toLowerCase() === "64 avg / 1sec in cc") {
                        config_c += '// Averaging : 64 Averages\n';
                        config_setting_c += '| 0x0060';
                    }
                
                    if (ti_widget_droplist_alertmode.selectedText.toLowerCase() === "alert mode") {
                        config_c += '// Alert Mode selected\n';
                    } else if (ti_widget_droplist_alertmode.selectedText.toLowerCase() === "thermal mode") {
                        config_c += '// Thermal Mode selected\n';
                        config_setting_c += '| 0x0010';
                    }

                    if (ti_widget_droplist_alertpinpolarity.selectedText.toLowerCase() === "active low") {
                        config_c += '// Active Low Polarity\n';
                    } else if (ti_widget_droplist_alertpinpolarity.selectedText.toLowerCase() === "active high") {
                        config_c += '// Active High Polarity\n';
                        config_setting_c += '| 0x0008';
                    }
                
                    if (ti_widget_droplist_alertpinoutput.selectedText.toLowerCase() === "alert") {
                        config_c += '// Alert pin Function\n';
                        config_setting_c += '\n';
                    } else if (ti_widget_droplist_alertpinoutput.selectedText.toLowerCase() === "data ready") {
                        config_c += '// Alert pin is Data Ready Function\n';
                        config_setting_c += '| 0x0004\n';
                    }
                
                    result_c += config_c + config_setting_c;
                }

                if(templateObj.$.ti_widget_code_high_limit.checked) {
                    var highlimit_in_c;
                    if(tempScale === "F") {
                        highlimit_in_c = getC(templateObj.$.ti_widget_spinner_high_limit_cfg.value);
                    } else {
                        highlimit_in_c = templateObj.$.ti_widget_spinner_high_limit_cfg.value;
                    }

                    var conv_value = Math.round(highlimit_in_c/0.0078125);
                
                    result_c += '// Hi Limit of ' + highlimit_in_c + ' C\n';
                    if(conv_value >= 0) {
                        result_c += '#define TMP117_0_HI_LIMIT_OFFSET ' + getHex16(conv_value) + '\n';
                    } else {
                        conv_value = 65536 + conv_value;
                        result_c += '#define TMP117_0_HI_LIMIT_OFFSET ' + getHex16(conv_value) + '\n';
                    }
                }
                
                if(templateObj.$.ti_widget_code_low_limit.checked) {
                    var lowlimit_in_c;
                    if(tempScale === "F") {
                        lowlimit_in_c = getC(templateObj.$.ti_widget_spinner_low_limit_cfg.value);
                    } else {
                        lowlimit_in_c = templateObj.$.ti_widget_spinner_low_limit_cfg.value;
                    }

                    conv_value = Math.round(lowlimit_in_c/0.0078125);

                    result_c += '// Lo Limit of ' + lowlimit_in_c + ' C\n';
                    if(conv_value >= 0) {
                        result_c += '#define TMP117_0_LO_LIMIT_OFFSET ' + getHex16(conv_value) + '\n';
                    } else {
                        conv_value = 65536 + conv_value;
                        result_c += '#define TMP117_0_LO_LIMIT_OFFSET ' + getHex16(conv_value) + '\n';
                    }
                }
                
                if(templateObj.$.ti_widget_code_eeprom1.checked) {
                    result_c += '#define TMP117_0_EEPROM1 ' + getHex16(templateObj.$.ti_widget_spinner_eeprom1.value) + '\n';
                }
                if(templateObj.$.ti_widget_code_eeprom2.checked) {
                    result_c += '#define TMP117_0_EEPROM2 ' + getHex16(templateObj.$.ti_widget_spinner_eeprom2.value) + '\n';
                }
                if(templateObj.$.ti_widget_code_eeprom3.checked) {
                    result_c += '#define TMP117_0_EEPROM3 ' + getHex16(templateObj.$.ti_widget_spinner_eeprom3.value) + '\n';
                }
                
                if(templateObj.$.ti_widget_code_temp_offset.checked) {
                    var tempoffset_in_c;
                    if(tempScale === "F") {
                        tempoffset_in_c = getC(templateObj.$.ti_widget_spinner_offset_temp.value);
                    } else {
                        tempoffset_in_c = templateObj.$.ti_widget_spinner_offset_temp.value;
                    }

                    conv_value = Math.round(tempoffset_in_c/0.0078125);

                    result_c += '// Temp Offset Limit of ' + tempoffset_in_c + ' C\n';
                    if(conv_value >= 0) {
                        result_c += '#define TMP117_0_TEMP_OFFSET ' + getHex16(conv_value) + '\n';
                    } else {
                        conv_value = 65536 + conv_value;
                        result_c += '#define TMP117_0_TEMP_OFFSET ' + getHex16(conv_value) + '\n';
                    }
                }
                
                result_c += '\nvoid sensor_write(unsigned long slaveaddress,';
                result_c += '\n                  unsigned long registerpointer,';
                result_c += '\n                  unsigned long registerdata);';
                result_c += '\nunsigned long sensor_read(unsigned long slaveaddress,';
                result_c += '\n                          unsigned long registerpointer);';
                result_c += '\nvoid tmp117_config(void);';
                result_c += '\nunsigned long tmp117_read(unsigned long sensorid);\n';
                saveText("tmp117.h", result_c);
            }); 

            // other event handlers added programmatically:
            // other event handlers added programmatically:
            templateObj.$.ti_widget_register_page.addEventListener("read_reg", function(event) {
                var detailsObj = event.detail;
                templateObj.logMsgAndNotify(detailsObj.statusType, detailsObj.statusMsg, false, true);
            }.bind(this));

            templateObj.$.ti_widget_register_page.addEventListener("read_regs", function(event) {
                var eventInfo = event.detail;
                templateObj.logMsgAndNotify(eventInfo.statusType, eventInfo.statusMsg, false, true);
            }.bind(this));

            templateObj.$.ti_widget_register_page.addEventListener("update_mode_changed", function(event) {
                var detailsObj = event.detail;
                templateObj.$.ti_widget_eventlog_view.log("info", "request to change update mode to " + detailsObj.mode.label);
            }.bind(this));

            templateObj.$.ti_widget_register_page.addEventListener("auto_read_config_changed", function(event) {
                var detailsObj = event.detail;
                templateObj.$.ti_widget_eventlog_view.log("info", "request to change automatic read interval to " + detailsObj.autoRead.label);
            }.bind(this));


            var usbHidService = gc.services['ti-service-usbhid'];
            if (usbHidService) {
                usbHidService.addEventListener('usbHidError', function(event) {
                    var detailsObj = event.detail;
                    templateObj.logMsgAndNotify("error", detailsObj.info + ": " + detailsObj.error, false, true);
                    gc.connectionManager.doDisconnect();
                })
            }



            templateObj.getRegisterData = function() {
                var regData = {
                    signature: "oneui-register-data",
                    data: []
                }
                var regBlocks = templateObj.$.ti_widget_register_page.regBlocks;
                for (var i = 0; i < regBlocks.length; i++) {
                    var registersData = [];
                    for (var r = 0; r < regBlocks[i].registers.length; r++) {
                        var reg = regBlocks[i].registers[r];
                        var regName = reg.name.toLowerCase();
                        regName = regName.replace(/\&/g, "_");
                        var item = {
                            idx: r,
                            id: regName,
                            value: reg.value
                        }
                        registersData.push(item);
                    }
                    regData.data.push(registersData);
                }
                var strJson = JSON.stringify(regData);
                return strJson;
            };

            templateObj.setRegisterData = function(strJson) {
                var regData = JSON.parse(strJson);
                var ok = true;
                var regBlocks = templateObj.$.ti_widget_register_page.regBlocks;
                for (var b = 0; b < regBlocks.length; b++) {
                    var registersData = [];
                    for (var r = 0; r < regBlocks[b].registers.length; r++) {
                        var reg = regBlocks[b].registers[r];
                        var regName = reg.name.toLowerCase();
                        regName = regName.replace(/\&/g, "_");
                        item = regData.data[b][r];
                        if (item.id === regName) {
                            reg.value = item.value;
                            templateObj.$.ti_widget_register_page.writeValueToTargetRegister(intValue, null, b, r);
                        } else {
                            var msg = "load registers failed: register name mismatch (" + item.id + " does not match " + regName + ")";
                            templateObj.logMsgAndNotify("warning", msg, false, true);
                        }
                    }

                }
            };

            templateObj.$.ti_widget_vtabcontainer.addEventListener("before_selected_tab_changed", function(event) {
                templateObj.$.ti_widget_temperaturegraph.ignoreNextResizeEvent = true;
            });
            
            templateObj.showToastMsg = function(msgType, msg, durationInSeconds) {
                templateObj.$.ti_widget_toast.hideToast();
                templateObj.$.ti_widget_toast.message = msg;
                switch (msgType.toLowerCase()) {
                    case "error":
                        templateObj.$.ti_widget_toast.iconName = "error";
                        templateObj.$.ti_widget_toast.backgroundColor = "red";
                        break;
                    case "warning":
                        templateObj.$.ti_widget_toast.iconName = "warning";
                        templateObj.$.ti_widget_toast.backgroundColor = "orange";
                        break;
                    default:
                        templateObj.$.ti_widget_toast.iconName = "check";
                        templateObj.$.ti_widget_toast.backgroundColor = "teal";
                        break;
                }
                templateObj.$.ti_widget_toast.duration = +durationInSeconds;
                templateObj.$.ti_widget_toast.showToast();
                console.log("toast msg [" + msgType + "]: " + msg);
            };

            templateObj.logMsgAndNotify = function(msgType, msg, showInStatusBar, showInToast) {
                var iconName = templateObj.$.ti_widget_eventlog_view.iconForItem(msgType);
                if (showInStatusBar) {
                    var statusBar = gc.services['ti-widget-statusbar'];
                    if (statusBar) {
                        statusBar.statusString3 = msg;
                        statusBar.setIcon(iconName);
                    }
                }
                var duration = 3;
                switch (msgType) {
                    case "warning":
                        duration = 4;
                        break;
                    case "error":
                        duration = 6;
                        break;
                    default:
                        duration = 3;
                        break;
                }
                if (showInToast) {
                    templateObj.showToastMsg(msgType, msg, duration);
                }
                templateObj.$.ti_widget_eventlog_view.log(msgType, msg);
            };
            Polymer.updateStyles();
        }, 1);

    });
};

templateObj = document.querySelector('#template_obj');
if (templateObj) {
    init();
} else {
    document.addEventListener('DOMContentLoaded', init.bind(this))
}